/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

 /**
 *\file CHistogram.cpp
 *\brief  Histogram class method development.
 */

#include "CHistogram.h"
#include <myUtils.h>
#include "TStyle.h"
#include <iostream>
#include <exception> 

IRenderer::Factory registerFactoryH("histogram", CHistogram::createObj);

class HistoException : public std::exception {
public:
        HistoException(const char *msg) throw() : errMessage(msg) { } ;
        ~HistoException(void) throw() { } ;
        const char *what(void) const throw() { return errMessage.c_str(); };
private:
        std::string errMessage;
       	HistoException(void) throw();
};

IRenderer* CHistogram::createObj(const void* node){
	return new CHistogram((const DOMNode*)node);
}

CHistogram::CHistogram(const DOMNode* node){
	parser_ = new CXMLParser;
	
	//function for graph presets xml parsing 
	histoPreset(node);
	
	histo_ = new TH1D(name_.c_str(), title_.c_str(), nBins_, min_, max_);			
	
	if((cTitle_ != "") && (cWidth_ != 0) && (cHeight_ != 0)){	
		setCanvas();	
	}else{	
		histo_->Draw();
	}
}

CHistogram::~CHistogram(){
	if(histo_) delete histo_;
	if(canv_) delete canv_;	
	if(cnode_) delete cnode_;
	if(hnode_) delete hnode_;
	if(parser_) delete parser_;
}

void CHistogram::histoPreset(const DOMNode* node){
	char ret[256];
	
	if(!parser_->FindNodeRootat((DOMNode*)node, (char*)"histogram_preset", hnode_)){
		throw HistoException("histogram_preset node isn't found");
	}else{
		if (!parser_->GetAttribute(hnode_, (char*)"histogram_name", (char*)ret)){
			throw HistoException("histogram_name attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Histogram name returned: %s\n", ret);
			#endif
			name_ = ret;		/* Histogram name*/
			IRenderer::checkName(name_);
		}
				
		if (!parser_->GetAttribute(hnode_, (char*)"histogram_title", (char*)ret)){
			throw HistoException("histogram_title attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Histogram title returned: %s\n", ret);
			#endif
			title_ = ret;		/* Histogram title*/
		}
		
		if (!parser_->GetAttribute(hnode_, (char*)"histogram_nbins", (char*)ret)){
			throw HistoException("histogram_nbins attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Histogram number of bins returned: %s\n", ret);
			#endif
			nBins_ = atol(ret);	/* Number of bins*/
		}
		
		if (!parser_->GetAttribute(hnode_, (char*)"histogram_min", (char*)ret)){
			throw HistoException("histogram_min attribute isn't found");
		}else{
			min_ = atof(ret);	/* Histogram min value*/
			#ifdef DEBUG
			printf("Histogram minimum value returned: %s\n", ret);
			#endif			
		}
		
		if (!parser_->GetAttribute(hnode_, (char*)"histogram_max", (char*)ret)){
			throw HistoException("histogram_max attribute isn't found");
		}else{
			max_ = atof(ret);	/* Histogram max value*/
			#ifdef DEBUG
			printf("Histogram maximum value returned: %s\n", ret);
			#endif		
		}
	}
	canvasPreset(node);
}

void CHistogram::canvasPreset(const DOMNode* node){
	char ret[256];

	if(parser_->FindNodeRootat((DOMNode*)node, (char*)"canvas_preset", cnode_)){		
		if (!parser_->GetAttribute(cnode_, (char*)"canvas_title", (char*)ret)){
			throw HistoException("canvas_title attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Canvas title: %s\n", ret);
			#endif
			cTitle_ = ret;		/* Canvas title*/
		}
		
		if (!parser_->GetAttribute(cnode_, (char*)"canvas_width", (char*)ret)){
			throw HistoException("canvas_width attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Canvas width: %s\n", ret);
			#endif
			cWidth_ = atoi(ret);	/* Canvas width (in pixels)*/
		}
		
		if (!parser_->GetAttribute(cnode_, (char*)"canvas_height", (char*)ret)){
			throw HistoException("canvas_height attribute isn't found");
		}else{
			#ifdef DEBUG
			printf("Canvas name: %s\n", ret);
			#endif
			cHeight_ = atoi(ret);	/* Canvas height (in pixels)*/
		}
	}else{
		cTitle_ = "";
		cWidth_ = 0;
		cHeight_ = 0;
	}
	
}

void CHistogram::setCanvas(void){
	std::string cName_ = name_;
	cName_.push_back((char)'_');
	
	#ifdef DEBUG
	cerr << "Canvas name : " << cName_ << endl;
	#endif

	//Instanciate a canvas
	canv_ = new TCanvas(cName_.c_str(), cTitle_.c_str(), cWidth_, cHeight_);

	canv_->SetFillColor(0);
	canv_->SetBorderMode(0);
	canv_->SetBorderSize(2);
	canv_->SetFrameBorderMode(0);
	canv_->SetHighLightColor(0);
	canv_->Draw();
	canv_->Update();
	histo_->Draw();
	canv_->cd();
}

void CHistogram::fillHisto(double data){
	#ifdef DEBUG
	std::cerr << "Fill histo: " << data << std::endl;
	#endif
	histo_->Fill((double)data);
}

void CHistogram::render(void* data, unsigned long size){
	switch(vType_){
		case T_INT8:{
			char dummy = *((char*)data);
			#ifdef DEBUG
			std::cerr << "Data to fill: " << (double)dummy << std::endl;
			#endif
			
			fillHisto((double)dummy);
		}
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
		case T_UINT8: {
			unsigned char dummy = *((unsigned char*)data);
			#ifdef DEBUG
			std::cerr << "Data to fill: " << (double)dummy << std::endl;
			#endif
			fillHisto((double)dummy);
		}	
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
			
		case T_INT16:{
			short dummy = *((short*)data);
			#ifdef DEBUG
			std::cerr << "Data to fill: " << dummy << std::endl;
			#endif
			fillHisto((double)dummy);
		}
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
			
		case T_UINT16:{
			unsigned short dummy = *((unsigned short*)data);
			#ifdef DEBUG
			std::cerr << "Data to fill: " << dummy << std::endl;
			#endif
			fillHisto((double)dummy);
		}	
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
			
		case T_INT32:{
			int dummy = *((int*)data);
			#ifdef DEBUG
			std::cerr << "Data to fill: " << dummy << std::endl;
			#endif
			fillHisto((double)dummy);;
		}
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
		
		case T_UINT32:{
			unsigned int dummy = *((unsigned int*)(data));
			#ifdef DEBUG
			std::cerr << "Data to fill: " << dummy << std::endl;
			#endif
			fillHisto((double)dummy);
		}
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
			
		case T_INT64:{
			long dummy = *((long*)data);
			#ifdef DEBUG
			std::cerr << "Data to fill: " << dummy << std::endl;
			#endif
			fillHisto((double)dummy);
		}
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
			
		case T_UINT64:{
			unsigned long dummy = *((unsigned long*)data);
			#ifdef DEBUG
			std::cerr << "Data to fill: " << dummy << std::endl;
			#endif
			fillHisto((double)dummy);
		}
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
		
		case T_FLOAT32:{
			float dummy = *((float*)data);
			#ifdef DEBUG
			std::cerr << "Data to fill: " << dummy << std::endl;
			#endif
			fillHisto((double)dummy);
		}
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
			
		case T_FLOAT64:{
			double dummy = *((double*)data);
			#ifdef DEBUG
			std::cerr << "Data to fill: " << dummy << std::endl;
			#endif
			fillHisto((double)dummy);
		}
		if(canv_){
			canv_->Modified();
			canv_->Update();
		}
		break;
		
		default:
			break;
	}

}
